!**************************************************************************************************
! Module/Program              | bufr_module
! Author                      | F. Guillaume (CNRM/GMAP/OBS)
! Creation Date               | 05/12/2012
!--------------------------------------------------------------------------------------------------
! Description : declarations and definitions for BUFR software. 
!--------------------------------------------------------------------------------------------------
! Global Variables : 
!--------------------------------------------------------------------------------------------------
! History :
!           16/06/2010 F. Guillaume       : 
!**************************************************************************************************

MODULE BUFR_MODULE

use parkind1, only : jpim, jprb

  implicit none


!-- BUFR variable

  real   (kind=jprb), parameter :: rvind = 1.7E38_jprb

  integer(kind=jpim), parameter :: nvind = 2147483647
  integer(kind=jpim), parameter :: jsup  =          9
  integer(kind=jpim), parameter :: jsec0 =          3
  integer(kind=jpim), parameter :: jsec1 =         40
  integer(kind=jpim), parameter :: jsec2 =       4096
  integer(kind=jpim), parameter :: jsec3 =          4
  integer(kind=jpim), parameter :: jsec4 =          2

#ifdef _BUFR_JELEM
  integer(kind=jpim), parameter :: jelem = _BUFR_JELEM
  integer(kind=jpim), parameter :: kelem = _BUFR_JELEM
#else
  integer(kind=jpim), parameter :: jelem = 160000
  integer(kind=jpim), parameter :: kelem = 160000
#endif

  integer(kind=jpim), parameter :: jbufl = 5000000
  integer(kind=jpim), parameter :: jbpw  =      32

#ifdef _BUFR_JWORK
  integer(kind=jpim), parameter :: jwork = _BUFR_JWORK
  integer(kind=jpim), parameter :: kvals = _BUFR_JWORK
#else
  integer(kind=jpim), parameter :: jwork = 4096000
  integer(kind=jpim), parameter :: kvals = 4096000
#endif

  integer(kind=jpim), parameter :: jkey  =      46
  integer(kind=jpim), parameter :: jbyte = 2400000

! paramètres du CEP

!integer(kind=jpim),PARAMETER ::JSUP =   9
!integer(kind=jpim),PARAMETER ::JSEC0=   3
!integer(kind=jpim),PARAMETER ::JSEC1= 40
!integer(kind=jpim),PARAMETER ::JSEC2=4096
!integer(kind=jpim),PARAMETER ::JSEC3=   4
!integer(kind=jpim),PARAMETER ::JSEC4=2
!integer(kind=jpim),PARAMETER ::JELEM=320000
!integer(kind=jpim),PARAMETER ::JSUBS=400
!integer(kind=jpim),PARAMETER ::JCVAL=150 
!integer(kind=jpim),PARAMETER ::JBUFL=512000
!integer(kind=jpim),PARAMETER ::JBPW =  64
!integer(kind=jpim),PARAMETER ::JTAB =3000
!integer(kind=jpim),PARAMETER ::JCTAB=3000
!integer(kind=jpim),PARAMETER ::JCTST=9000
!integer(kind=jpim),PARAMETER ::JCTEXT=9000
!integer(kind=jpim),PARAMETER ::JWORK=4096000
!integer(kind=jpim),PARAMETER ::JKEY=46
!integer(kind=jpim),PARAMETER ::JTMAX=10
!integer(kind=jpim),PARAMETER ::JTCLAS=64
!integer(kind=jpim),PARAMETER ::JTEL=255
!integer(kind=jpim),PARAMETER ::KELEM=320000
!integer(kind=jpim),PARAMETER ::KVALS=4096000
!integer(kind=jpim),PARAMETER ::jbyte=512000


  integer(kind=jpim), dimension(jsup)  :: ksup
  integer(kind=jpim), dimension(jsec0) :: ksec0
  integer(kind=jpim), dimension(jsec1) :: ksec1
  integer(kind=jpim), dimension(jsec2) :: ksec2
  integer(kind=jpim), dimension(jsec3) :: ksec3
  integer(kind=jpim), dimension(jsec4) :: ksec4
  integer(kind=jpim), dimension(jkey)  :: key

  integer(kind=jpim) :: kbox
  integer(kind=jpim) :: kbufl
  integer(kind=jpim) :: ktdlen
  integer(kind=jpim) :: ktdexl
  integer(kind=jpim) :: kapp
  integer(kind=jpim) :: klen
  integer(kind=jpim) :: krql

  integer(kind=jpim), dimension(:), allocatable :: kbuff, kbufr, ktdlst, ktdexp
  integer(kind=jpim), dimension(:), allocatable :: kdata, kboxr, krq,    kreq

  real(kind=jprb), dimension(:), allocatable    :: values, vals, rqv

  character(len=64), dimension(:), allocatable  :: cnames, cname
  character(len=24), dimension(:), allocatable  :: cunits, cunit
  character(len=80), dimension(:), allocatable  :: cvals
  character(len=24), dimension(:), allocatable  :: cboxu
  character(len=64), dimension(:), allocatable  :: cboxn


contains


!**************************************************************************************************
! Subroutine/Function | bufr_alloc
! Author              | 
! Creation Date       | 
!--------------------------------------------------------------------------------------------------
! Description :  
!--------------------------------------------------------------------------------------------------
! History : 
!**************************************************************************************************
subroutine bufr_alloc

  implicit none
  
  integer(kind=jpim) :: iret

  allocate(kbuff(jbufl),stat=iret)
  if (iret /= 0) stop 'bufr_alloc() COULD NOT allocate kbuff(:)'
  allocate(kbufr(jbufl),stat=iret)
  if (iret /= 0) stop 'bufr_alloc() COULD NOT allocate kbufr(:)'
  allocate(values(kvals),stat=iret)
  if (iret /= 0) stop 'bufr_alloc() COULD NOT allocate values(:)'
  allocate(vals(kvals),stat=iret)
  if (iret /= 0) stop 'bufr_alloc() COULD NOT allocate vals(:)'
  allocate(kboxr(kvals),stat=iret)
  if (iret /= 0) stop 'bufr_alloc() COULD NOT allocate kboxr(:)'
  allocate(cvals(kvals),stat=iret)
  if (iret /= 0) stop 'bufr_alloc() COULD NOT allocate cvals(:)'
  allocate(ktdlst(kelem),stat=iret)
  if (iret /= 0) stop 'bufr_alloc() COULD NOT allocate ktdlst(:)'
  allocate(ktdexp(kelem),stat=iret)
  if (iret /= 0) stop 'bufr_alloc() COULD NOT allocate ktdexp(:)'
  allocate(kdata(kelem),stat=iret)
  if (iret /= 0) stop 'bufr_alloc() COULD NOT allocate kdata(:)'
  allocate(krq(kelem),stat=iret)
  if (iret /= 0) stop 'bufr_alloc() COULD NOT allocate krq(:)'
  allocate(kreq(kelem),stat=iret)
  if (iret /= 0) stop 'bufr_alloc() COULD NOT allocate kreq(:)'
  allocate(rqv(kelem),stat=iret)
  if (iret /= 0) stop 'bufr_alloc() COULD NOT allocate rqv(:)'
  allocate(cnames(kelem),stat=iret)
  if (iret /= 0) stop 'bufr_alloc() COULD NOT allocate cnames(:)'
  allocate(cname(kelem),stat=iret)
  if (iret /= 0) stop 'bufr_alloc() COULD NOT allocate cname(:)'
  allocate(cunits(kelem),stat=iret)
  if (iret /= 0) stop 'bufr_alloc() COULD NOT allocate cunits(:)'
  allocate(cunit(kelem),stat=iret)
  if (iret /= 0) stop 'bufr_alloc() COULD NOT allocate cunit(:)'
  allocate(cboxu(kelem),stat=iret)
  if (iret /= 0) stop 'bufr_alloc() COULD NOT allocate cboxu(:)'
  allocate(cboxn(kelem),stat=iret)
  if (iret /= 0) stop 'bufr_alloc() COULD NOT allocate cboxn(:)'

end subroutine bufr_alloc

END MODULE BUFR_MODULE
