!**************************************************************************************************
! Program                     | DecodBufr V.1.02
! Author                      | F. Guillaume (CNRM/GMAP/OBS)
! Creation Date               | 06/06/2016
!--------------------------------------------------------------------------------------------------
! Description : decod bufr files
!--------------------------------------------------------------------------------------------------
! Global Variables : 
!--------------------------------------------------------------------------------------------------
! History :
!           06/06/2016 : first release
!           17/06/2016 : better error diagnostics
!           22/06/2016 : allow to disable expanded and/or unexpanded data descriptors printing
!**************************************************************************************************
PROGRAM DecodBufr

use            :: parkind1,     only : jpim, jpib, jprb
use            :: bufr_module
use            :: decodtool_module

 character(len=256)              :: InputFile, chaine
 character(len=256),dimension(9) :: Args
 integer(kind=jpim)              :: nulbufr, iret, NbArgs
 integer(kind=jpim)              :: length, NumFic, Version, kel, i, pos
 integer(kind=jpim)              :: FirstFile, LastFile, FirstSubset, LastSubset
 logical                         :: ParamBator, LPrtSec0, LPrtSec1, LPrtSec2, LPrtSec3, LPrtSec3e, LPrtSec4

! initialization
call bufr_alloc()

nulbufr      = 0
iret         = 0
Numfic       = 0
Args(:)      = ''
ParamBator   = .FALSE.
FirstFile    = 0
LastFile     = 0
FirstSubset  = 0
LastSubset   = 0
LPrtSec0     = .TRUE.
LPrtSec1     = .TRUE.
LPrtSec2     = .TRUE.
LPrtSec3     = .TRUE.
LPrtSec3e    = .TRUE.
LPrtSec4     = .TRUE.

NbArgs = command_argument_count()
if (NbArgs > size(Args)) call erreur("***** ERROR : too many arguments")

do i=1, NbArgs
  call get_command_argument(i,Args(i),length,iret)
enddo

i = 1
if (NbArgs < 1) call erreur("") 
do while (i <= NbArgs)
  select case (trim(Args(i)))
  case ('-i') ! input file
    InputFile = Args(i+1)
    i = i+2
  case ('-f') ! numeros des fichiers a traiter
    chaine = Args(i+1)
    pos = index(chaine,'-')
    if (pos > 0) then
      read(chaine(1:pos-1),*,iostat=iret) FirstFile
      if (iret /= 0) call erreur("***** ERROR : bad values in -f option")
      read(chaine(pos+1:len(trim(chaine))),*,iostat=iret) LastFile
      if (iret /= 0) call erreur("***** ERROR : bad values in -f option")
      if (FirstFile > LastFile .or. FirstFile <= 0) call erreur("***** ERROR : bad values in -f option")
    else
      read(chaine,*,iostat=iret) FirstFile
      if (iret /= 0 .or. FirstFile <= 0) call erreur("ERROR : bad values in -f option")
      LastFile = FirstFile
    endif
    i = i+2
  case ('-s') ! numeros des subsets à traiter
    chaine = Args(i+1)
    pos = index(chaine,'-')
    if (pos > 0) then
      read(chaine(1:pos-1),*,iostat=iret) FirstSubset
      if (iret /= 0) call erreur("***** ERROR : bad values in -s option")
      read(chaine(pos+1:len(trim(chaine))),*,iostat=iret) LastSubset
      if (iret /= 0) call erreur("***** ERROR : bad values in -s option")
      if (FirstSubset > LastSubset .or. FirstSubset <= 0) call erreur("***** ERROR : bad values in -s option")
    else
      read(chaine,*,iostat=iret) FirstSubset
      if (iret /= 0 .or. FirstSubset <= 0) call erreur("***** ERROR : bad values in -s option")
      LastSubset = FirstSubset
    endif
    i = i+2
  case ('-b') ! sortie mode Bator
    ParamBator = .TRUE.
    i = i+1
  case ('-h') ! sections a supprimer de l'affichage
    chaine = Args(i+1)
    if (index(chaine,"0") >0) LPrtSec0  = .FALSE.
    if (index(chaine,"1") >0) LPrtSec1  = .FALSE.
    if (index(chaine,"2") >0) LPrtSec2  = .FALSE.
    if (index(chaine,"3") >0) LPrtSec3  = .FALSE.
    if (index(chaine,"e") >0) LPrtSec3e = .FALSE.
    if (index(chaine,"4") >0) LPrtSec4  = .FALSE.
    i = i+2
  case default
    call erreur("***** ERROR : bad arguments")
  end select
enddo

call pbopen(nulbufr,InputFile,'r',iret)
if (iret /=0 ) call erreur("***** ERROR : can't open input file")

call CountBufr(nulbufr,FirstFile,LastFile,FirstSubset,LastSubset)
call pbseek(nulbufr,0,0,iret)

Numfic = FirstFile - 1

do i=1, FirstFile - 1
  call pbbufr(nulbufr,kbuff,jbyte,kbufl,iret)
  if (iret /= 0) call erreur("***** ERROR : can't read bufr file")
enddo

do i=FirstFile, LastFile
  call pbbufr(nulbufr,kbuff,jbyte,kbufl,iret)
  iword  = kbufl / (jbpw / 8) + 1
  Numfic = NumFic + 1
  call bus012(iword,kbuff,ksup,ksec0,ksec1,ksec2,iret)
  if (iret == 0 ) then
    call PrintSec0(NumFic,Version,LPrtSec0)
    call PrintSec1(Version,LPrtSec1)
    kel = jwork / ksup(6)
    call bufrex(kbufl,kbuff,ksup,ksec0,ksec1,ksec2,ksec3,ksec4, &
             & kel,cnames,cunits,jwork,values,cvals,iret)
    if (iret == 0) then
      if (ksec1(5) /= 0) then
        call bfkey(ksec1,ksec2,key,ksup,iret)
        if (iret == 0 .and. LPrtSec2) call bfprs2(ksup ,key)
      endif
      call busel(ktdlen,ktdlst,ktdexl,ktdexp,iret)
      call PrintSec3(ParamBator,LPrtSec3,LPrtSec3e)
      call PrintSec4(kel,FirstSubset,LastSubset,LPrtSec4)
    endif
  endif
enddo
select case (iret)
case (-1)
  write(*,'("Fin de fichier - lecture OK")')
end select
call pbclose(nulbufr,iret)

END PROGRAM DecodBufr
